"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgV4SimpleSubscriptionsPlugin = void 0;
const json_1 = require("@dataplan/json");
const grafast_1 = require("grafast");
const graphile_utils_1 = require("graphile-utils");
exports.PgV4SimpleSubscriptionsPlugin = (0, graphile_utils_1.makeExtendSchemaPlugin)((build) => {
    const nodeIdHandlerByTypeName = build.getNodeIdHandlerByTypeName?.();
    return {
        typeDefs: [
            (0, graphile_utils_1.gql) `
        extend type Subscription {
          listen(topic: String!): ListenPayload
        }

        type ListenPayload {
          event: String
        }
      `,
            ... // Only add the relatedNode if supported
            (nodeIdHandlerByTypeName
                ? [
                    (0, graphile_utils_1.gql) `
              extend type ListenPayload {
                relatedNode: Node
                relatedNodeId: ID
              }
            `,
                ]
                : []),
        ],
        plans: {
            Subscription: {
                listen: {
                    subscribePlan: (0, graphile_utils_1.EXPORTABLE)((context, jsonParse, lambda, listen) => function subscribePlan(_$root, { $topic }) {
                        const $pgSubscriber = context().get("pgSubscriber");
                        const $derivedTopic = lambda($topic, (topic) => `postgraphile:${topic}`);
                        return listen($pgSubscriber, $derivedTopic, jsonParse);
                    }, [grafast_1.context, json_1.jsonParse, grafast_1.lambda, grafast_1.listen]),
                    plan: (0, graphile_utils_1.EXPORTABLE)(() => function plan($event) {
                        return $event;
                    }, []),
                },
            },
            ListenPayload: {
                event: (0, graphile_utils_1.EXPORTABLE)(() => ($event) => {
                    return $event.get("event");
                }, []),
                ...(nodeIdHandlerByTypeName
                    ? {
                        relatedNodeId: (0, graphile_utils_1.EXPORTABLE)((nodeIdFromEvent) => ($event) => {
                            return nodeIdFromEvent($event);
                        }, [nodeIdFromEvent]),
                        relatedNode: (0, graphile_utils_1.EXPORTABLE)((node, nodeIdFromEvent, nodeIdHandlerByTypeName) => ($event) => {
                            const $nodeId = nodeIdFromEvent($event);
                            return node(nodeIdHandlerByTypeName, $nodeId);
                        }, [grafast_1.node, nodeIdFromEvent, nodeIdHandlerByTypeName]),
                    }
                    : null),
            },
        },
    };
}, "PgV4SimpleSubscriptionsPlugin");
// base64JSON codec copy
const nodeObjToNodeId = (0, graphile_utils_1.EXPORTABLE)(() => function nodeObjToNodeId(obj) {
    if (!obj)
        return null;
    return Buffer.from(JSON.stringify(obj), "utf8").toString("base64");
}, [], "nodeObjToNodeId");
// WARNING: this function assumes that you're using the `base64JSON` NodeID codec, which was the case for PostGraphile V4. If you are not doing so, YMMV.
const nodeIdFromEvent = (0, graphile_utils_1.EXPORTABLE)((lambda, nodeObjToNodeId) => function nodeIdFromEvent($event) {
    const $nodeObj = $event.get("__node__");
    const $nodeId = lambda($nodeObj, nodeObjToNodeId);
    return $nodeId;
}, [grafast_1.lambda, nodeObjToNodeId], "nodeIdFromEvent");
//# sourceMappingURL=PgV4SimpleSubscriptionsPlugin.js.map